*Version 1.0
 Written by Carrie Walker;

***************************************************************************************
Instructions:
Download the 2023 Dialysis Event MRAT data from REDCap by using the 'Data Exports, Reports, and Stats' option
on the left-hand menu. Be sure to export data from the Dialysis Event instument only, and as a .csv 
format using variable names, not labels. Save the file in a secure location and enter the filepath below.

This code works with 2023 Dialysis Event REDCap data only.

For step by step instructions on exporting data, follow steps 1-5 of the How-To document located
at the link below. You DO NOT need to deidentify your data for this code and report. Please ignore
that step in the guidance document.
	https://www.cdc.gov/nhsn/pdfs/validation/2022/export-data-from-REDCap.pdf

Enter the facility's name where prompted. Do not use quotes, and be sure the semicolons 
aren't deleted.

Do not edit anywhere else in the code after these macros.
***************************************************************************************;

*Enter the filepath, including the file name and extension;
%let path = C:\filepath\2023_Dialysis_dataset.csv;

*Enter your jurisdiction's name, exactly as you want it written in the report.;
%let jurname = State Health Department;

proc import datafile="&path" 
out=rcde (where=(orgid_de_2023 ne .))
dbms=csv replace;
guessingrows=2000;
run;

proc format;
value ivammc
1="I) Misinterpretation of antibiotic administration or reporting criteria"
2="II) Missing data elements or incorrect information reported"
3="III) Other, specify";
value $ ivammcI 
"a"="a) A one-time or prophylactic dose was given"
"b"="b) Related blood culture results were negative"
"c"="c) IVAMS is for non-vascular access problem (e.g., pneumonia)"
"d"="d) First outpatient IVAM administration resulting from continuation of inpatient treatment" 
"e"="e) Administered on outpatient basis, outside of dialysis facility (e.g., LTC, prison)" 
"f"="f) Facility thought they were only supposed to report specific microbials (e.g., vanco)" 
"g"="g) Misapplication of 21-day rule; event was within 21 days of previous IVAM event"
"h"="h) Reported for an oral antibiotic" 
"i"="i) Not reported when drug not owned by clinic"
"j"="j) Not reported for deceased patients or patients transferred out of facility";
value $ ivammcII
"a"= "a) Did not report all event elements (e.g., reported PBC but not IVAM)" 
"b"= "b) Event Date incorrect" 
"c"= "c) Vascular Access type incorrect" 
"d"= "d) Other, specify";

value pbcmc
1="I) Misinterpretation of PBC reporting criteria"
2="II) Missing data elements or incorrect information reported"
3="III) Other, specify";
value $ pbcmcI 
"a"= "a) Source of infection reported as contamination or uncertain" 
"b"= "b) IV antimicrobials were not started" 
"c"= "c) Attributed to non-vascular access problems (e.g., wound, pneumonia)" 
"d"= "d) Obtained within first day after hospital admit" 
"e"= "e) Misapplication of 21-day rule; event was within 21 days of previous PBC event" 
"f"= "f) Not reported for deceased patients or patients transferred out of facility";
value $ pbcmcII 
"a"= "a) Did not report all event elements (e.g., reported IVAM but not PBC)" 
"b"= "b) Event Date incorrect" 
"c"= "c) Vascular Access type incorrect" 
"d"= "d) Other, specify";

value prsmc
1="I) Misinterpretation of PRS identification or reporting criteria"
2="II) Missing data elements or incorrect information reported"
3="III) Other, specify";
value $ prsmcI 
"a"="a) Not reported because antimicrobials were not started" 
"b"="b) Infection suspected but not confirmed and/or doctor says PRS is cellulitis" 
"c"="c) Facility lacks process to track PRS events" 
"d"="d) Misapplication of 21-day rule; event was within 21 days of previous PRS event" 
"e"="e) Not reported for deceased patients or patients transferred out of facility";
value $ prsmcII 
"a"="a) Did not report all event elements (e.g., reported IVAM but not PRS)" 
"b"="b) Event Date incorrect" 
"c"="c) Vascular Access type incorrect" 
"d"="d) Other, specify";
value resultstat
1="True Positive"
2="False Negative"
3="False Positive"
4="True Negative" 
9="Review";
value yesno
1="Yes"
0="No";
run;

data outcome_de;
set rcde ;
keep orgid_de: record_id patientid_de: ivam: pbc: prs: num:;
run;

proc sort data=outcome_de; by orgid_de_2023 record_id patientid_de_2023; run;
proc transpose data=outcome_de out=trans_de;
by orgid_de_2023 record_id patientid_de_2023;
var ivam: pbc: prs: ;
run;
proc sort data=trans_de; by record_id _name_; run;

data de2;
set trans_de;
length Drug $50. pathogen _mcreas2oth _mcreas3 $100.;
rename orgid_de_2023=orgid_de patientid_de_2023=patientid_de;
col2=strip(col1);
if col2 in ("." "") then delete;
if index(_name_,"prob_de") then delete;
if (index(_name_, "varep") > 0 or index(_name_, "vapres") > 0 or index(_name_, "accaff") > 0 or index(_name_, "sx_de") > 0) and col2="0" then delete;

if index(_name_,"drug_de") > 0 then Drug=col2;
if index(_name_,"startdt") > 0 then startdt=input(col2,mmddyy10.);
if index(_name_, "path") > 0 then pathogen=col2;
if index(_name_,"colldt") > 0 then Colldt=input(col2,mmddyy10.);
if index(_name_,"onsetdt") > 0 then onsetdt=input(col2,mmddyy10.);

if index(_name_,"reportable") > 0 then reportable=input(col2,1.);
if index(_name_,"reported") > 0 then reported=input(col2,1.);
if index(_name_,"repeventdt") > 0 then repdt=input(col2,mmddyy10.);

if index(_name_, "vapres_de_2023___f") > 0 and col2="1" then _VApresF="Fistula";
if index(_name_, "vapres_de_2023___g") > 0 and col2="1" then _VApresG="Graft";
if index(_name_, "vapres_de_2023___t") > 0 and col2="1" then _VApresT="Tunneled CL";
if index(_name_, "vapres_de_2023___nt") > 0 and col2="1" then _VApresNT="Non-Tunneled CL";
if index(_name_, "vapres_de_2023___o") > 0 and col2="1" then _VApresO="Other";
if index(_name_, "vapres_de_2023___na") > 0 and col2="1" then _VApresNA="None";

if index(_name_, "varep_de_2023___f") > 0 and col2="1" then _VArepF="Fistula";
if index(_name_, "varep_de_2023___g") > 0 and col2="1" then _VArepG="Graft";
if index(_name_, "varep_de_2023___t") > 0 and col2="1" then _VArepT="Tunneled CL";
if index(_name_, "varep_de_2023___nt") > 0 and col2="1" then _VArepNT="Non-Tunneled CL";
if index(_name_, "varep_de_2023___o") > 0 and col2="1" then _VArepO="Other";
if index(_name_, "varep_de_2023___na") > 0 and col2="1" then _VArepNA="None";

if index(_name_, "accaff_de_2023___f") > 0 and col2="1" then _VAaffF="Fistula";
if index(_name_, "accaff_de_2023___g") > 0 and col2="1" then _VAaffG="Graft";
if index(_name_, "accaff_de_2023___t") > 0 and col2="1" then _VAaffT="Tunneled CL";
if index(_name_, "accaff_de_2023___nt") > 0 and col2="1" then _VAaffNT="Non-Tunneled CL";
if index(_name_, "accaff_de_2023___o") > 0 and col2="1" then _VAaffO="Other";

if index(_name_, "sx_de_2023___p") > 0 and col2="1" then sx_p="Pus";
if index(_name_, "sx_de_2023___r") > 0 and col2="1" then sx_r="Redness";
if index(_name_, "sx_de_2023___s") > 0 and col2="1" then sx_s="Swelling";

if substr(_name_,1,4) =  "ivam" then do;
if index(_name_, "misclassreason") > 0 and col2 in ("1" "2" "3") then _mcreason=put(input(col2,1.), ivammc.);
if index(_name_, "misreason1") > 0 and col2 ne "" then _mcreas1=put(col2, ivammcI.);
if index(_name_, "misreason2") > 0 and col2 ne "" then _mcreas2=put(col2, ivammcII.);
if index(_name_, "misreason2_oth") > 0 and col2 ne "" then _mcreas2oth=col2;
if index(_name_, "misreason3") > 0 and col2 ne "" then _mcreas3=col2;
if index(_name_, "ivam10") > 0 then _ivamnum=10;
else _ivamnum=input(substr(_name_,5,1), 2.);end;

if substr(_name_,1,3) =  "pbc" then do;
if index(_name_, "misclassreason") > 0 and col2 in ("1" "2" "3") then _mcreason=put(input(col2,1.), pbcmc.);
if index(_name_, "misreason1") > 0 and col2 ne "" then _mcreas1=put(col2, pbcmcI.);
if index(_name_, "misreason2") > 0 and col2 ne "" then _mcreas2=put(col2, pbcmcII.);
if index(_name_, "misreason2_oth") > 0 and col2 ne "" then _mcreas2oth=col2;
if index(_name_, "misreason3") > 0 and col2 ne "" then _mcreas3=col2;
if index(_name_, "pbc10") > 0 then _pbcnum=10;
else _pbcnum=input(substr(_name_,4,1), 2.); end;

if substr(_name_,1,3) =  "prs" then do;
if index(_name_, "misclassreason") > 0 and col2 in ("1" "2" "3") then _mcreason=put(input(col2,1.), prsmc.);
if index(_name_, "misreason1") > 0 and col2 ne "" then _mcreas1=put(col2, prsmcI.);
if index(_name_, "misreason2") > 0 and col2 ne "" then _mcreas2=put(col2, prsmcII.);
if index(_name_, "misreason2_oth") > 0 and col2 ne "" then _mcreas2oth=col2;
if index(_name_, "misreason3") > 0 and col2 ne "" then _mcreas3=col2; 
if index(_name_, "prs10") > 0 then _prsnum=10;
else _prsnum=input(substr(_name_,4,1), 2.); end;

format startdt colldt onsetdt repdt mmddyy10.;
run;

proc sql; create table de3 as select orgid_de, patientid_de, _ivamnum, 
_pbcnum, _prsnum, 
max(Drug) as Drug, max(pathogen) as pathogen, max(colldt) as colldt format mmddyy10.,
max(startdt) as startdt format mmddyy10., max(onsetdt) as onsetdt format mmddyy10.,
max(reportable) as reportable, max(reported) as reported, max(repdt) as repdt format mmddyy10., 
max(_vapresf) as _vapresf, max(_vapresg) as _vapresg, max(_vaprest) as _vaprest, max(_vapresnt) as _vapresnt,
max(_vapreso) as _vapreso, max(_vapresna) as _vapresna, 
max(_varepf) as _varepf, max(_varepg) as _varepg, max(_varept) as _varept, max(_varepnt) as _varepnt,
max(_varepo) as _varepo, max(_varepna) as _varepna, 
max(_vaafff) as _vaafff, max(_vaaffg) as _vaaffg, max(_vaafft) as _vaafft, max(_vaaffnt) as _vaaffnt,
max(_vaaffo) as _vaaffo, max(sx_p) as sx_p, max(sx_r) as sx_r, max(sx_s) as sx_s,
max(_mcreason) as _mcreason, max(_mcreas1) as _mcreas1, max(_mcreas2) as _mcreas2, 
max(_mcreas2oth) as _mcreas2oth, max(_mcreas3) as _mcreas3
from de2
group by orgid_de, patientid_de, _ivamnum, _pbcnum, _prsnum;
quit;

data de4 needsreview;
set de3 (rename= ( _ivamnum = IVAMnumber _pbcnum = PBCnumber _prsnum = PRSnumber));
length VAaffected VApresent VAreported SXreported $50. pathogen $100. mcrep misclassreason $200.;
VAaffected=catx(", ", of _vaaff:);
	if VAaffected = "" then VAaffected="n/a";
VApresent=catx(", ", of _vapres:);
	if VApresent = "" then VApresent="n/a";
VAreported=catx(", ", of _varep:);
	if VAreported = "" then VAreported="n/a";

SXreported=catx(", ", of sx_:);
misclassreason=catx(": ", of _mcreas:);

*Creating a shortened misclassreason for report;
if index(misclassreason,"Other, specify") > 0 then mcrep = "Other or Unknown";
if index(misclassreason, "(III) Additional Reasons")>0 then mcrep=left(scan(misclassreason,-1,")"));
else if mcrep ne "Other or Unknown" then mcrep=left(scan(misclassreason,-1,")"));
if mcrep = "Other or Unknown" then mcrep_moreinfo=left(scan(misclassreason,-1,":"));

if ivamnumber ne . then EventType="IVAM";
if pbcnumber ne . then EventType="PBC";
if prsnumber ne . then EventType="PRS";

if drug = "" and startdt = . and reportable = . and reported = . then delete;
if pathogen = "" and colldt = . and reportable = . and reported = . then delete;
if onsetdt = . and reportable = . and reported = . then delete;

*Creating tp/fp/tn/fn;
if reportable=1 and reported=1 then do;
	Validator=1; Facility=1; Resultstatus=1; end; *TP;
if reportable=0 and reported=0 then do;
	Validator=0; Facility=0; Resultstatus=4; end; *TN;
if reportable=1 and reported=0 then do;
	Validator=1; Facility=0; Resultstatus=2; end; *FN;
if reportable=0 and reported=1 then do;
	Validator=0; Facility=1; Resultstatus=3; end; *FP;
if reportable in (1 0) and reported=. then Resultstatus=9; *review;
if reportable=. and reported in (1 0) then Resultstatus=9; *review;

*Creating casedet for report;
if resultstatus in (1 4) then casedet="Correctly Reported or Correctly Not Reported";
if resultstatus = 2 then casedet="Incorrectly NOT Reported/Under Reported";
if resultstatus = 3 then casedet="Incorrectly Reported/Over Reported";


label orgid_de= "OrgID" patientid_de="Patient ID" reportable='Reportable to NHSN' 
pathogen="Pathogen(s) Identified" colldt="Specimen Collection Date" onsetdt="Symptom Onset Date" 
reported='Reported by Facility' drug="IVAM Drug Name" startdt="IVAM Start Date" 
repdt="Report Date" vapresent="VA Present per chart" vareported="VA Reported to NHSN" VAaffected="VA Affected" 
misclassreason="Misclassification Reason(s)" SXreported="Reported Symptoms";
format reportable reported yesno. resultstatus resultstat.;
drop _:; label;

if resultstatus in (1:4) then output de4;
if resultstatus=9 then output needsreview;
run;

proc sort data=de4; by EventType descending Facility descending Validator; run;
proc freq data=de4 order=data;
by EventType; 
table facility*validator/nocol nocum norow nopercent out=de_freqsevtype;
format facility Validator yesno.;
run;
proc freq data=de4 order=data;
table facility*validator/nocol nocum norow nopercent out=de_freqstotal;
format facility Validator yesno.;
run;

data de_freqs;length eventtype fac $5.;
set de_freqsevtype de_freqstotal;
if eventtype="" then eventtype="Total";
percent2=round(percent, 1.);
fac=put(facility, yesno.);
drop percent; rename percent2 =percent;
run;

*Getting counts for case determination and misclassification reasons;
proc freq data=de4;
where Facility ne Validator;
table mcrep / out=misclass_de;
run;

proc freq data=de4;
table casedet / out=casedet_de;
run;
proc sort data=casedet_de; by descending count; run;
proc sort data=misclass_de; by descending count; run;

data misclass2_de;length Category Measure $250.;
set misclass_de;
Percent=round(percent, .01);
Category="Reason(s) for Disagreement^";
Measure=mcrep;
run;
data casedet2_de;length Category Measure $250.;
set casedet_de;
Percent=round(percent, .01);
Category="Case Determination Outcome";
Measure=casedet;
drop casedet;
run;

data Measures_cdmc_de;
set casedet2_de misclass2_de;
run;

*Calculating percent agreement;
proc sql; create table de_peragreement as select eventtype,
sum((case when facility=1 and validator=1 then count else 0 end) + 
		(case when facility=0 and validator=0 then count else 0 end)) as numerator,
sum(count) as denom, put(round(calculated numerator/(calculated denom)*100,.1),4.1) as PerAgree
from de_freqs 
group by eventtype;

create table de_orgs as select count(distinct(orgid_de)) as numorgs
from de4;
quit;

data _null_;
set de_peragreement;
if eventtype = "IVAM" then call symput ('ivampa', peragree);
if eventtype = "PBC" then call symput ('pbcpa', peragree);
if eventtype = "PRS" then call symput ('prspa', peragree);
if eventtype = "Total" then call symput ('overallpa', peragree);
call scan("&path", -1, pos, length,"\");
call symput("reppath", substr("&path",1,pos-2) || "\Dialysis - &jurname Validation Summary Report %sysfunc(date(),mmddyyd10.).xlsx");
run; 
%put &ivampa &pbcpa &prspa &overallpa;
%put &reppath;


*Assigning macros to get counts for overall agreement, NOT by event type;
%let a= 0;
%let b= 0;
%let c= 0;
%let d= 0;

data _null_;
set de_freqstotal;
if facility=1 and validator=1 then call symput("a",count);
if facility=0 and validator=0 then call symput("d",count);
if facility=1 and validator=0 then call symput("b",count);
if facility=0 and validator=1 then call symput("c",count);
run;

data _null_;
if &a = "" then call symput("a",0);
if &d = "" then call symput("d",0);
if &b = "" then call symput("b",0);
if &c = "" then call symput("c",0);
run;
%put &a;
%put &b;
%put &c;
%put &d;

data counts_de;
 a = &a; *TP;
 b = &b; *FP;
 c = &c; *FN;
 d = &d; *TN;
 N=a+b+c+d;
PerFalseRepd=round((b/n * 100),.01);
PerCorrNotRepd=round((d/n * 100),.01);
TotalHAIs=a + c;
PerHAI=round((TotalHAIs/n * 100),.01);
PerHAIcorrectrepd=round((a/TotalHAIs * 100),.01);
PerHAImissed=round(c/TotalHAIs * 100,.01);
NumDisagree=b+c;
PerDisagree=round((NumDisagree/n * 100),.01);
run;

proc transpose data=counts_de out=counts_trans_de;
run;

*Organizing & grouping counts/percents;
proc format;
value measure
1="Total Participating Facilities"
2="Total Cases Reviewed"
3="Facility-Validator Agree"
4="Facility-Validator Disagree"
8="Incorrectly Reported as HAI"
9="Correctly Not Reported as HAI"
10="Total HAIs Identified by Validator"
11="Correctly Reported by Facility*"
12="Missed/Not Reported by Facility*";
run;

data counts_trans_de2;
set counts_trans_de (rename=(_name_=Measure col1=count));
if measure in ("numorgs") then id=1;
if measure in ("TotalReviewed") then id=2;
if measure in ("NumAgreed" "PerAgree") then id=3;
if measure in ("NumDisagree" "PerDisagree") then id=4;
if measure in ("b" "PerFalseRepd") then id=8;
if measure in ("d" "PerCorrNotRepd") then id=9;
if measure in ("TotalHAIs" "PerHAI") then id=10;
if measure in ("a" "PerHAIcorrectrepd") then id=11;
if measure in ("c" "PerHAImissed") then id=12;
if id=. then delete;
format id measure.;
run;

proc sort data=counts_trans_de2; by id; run;
proc transpose data=counts_trans_de2 
out=counts_de2 (rename=(col1=Count col2=Percent) drop=_name_);
by id; 
var count;run;

data counts_de_final;length Category Measure $250.;
set counts_de2;
if id = 1 then category="Participating Facilities";
if id in (2 3 4) then category="Overall Agreement Status";
if id = 8 then category="Cases Incorrectly Reported as HAI";
if id = 9 then category="Cases Correctly Not Reported as HAI";
if id in (10 11 12) then category="Identified HAIs by Reporting Status";
Measure=put(id, measure.);
drop id;
run;

*Appending both counts datasets, prepping for report;
data forreport_de;
set counts_de_final measures_cdmc_de;
if category = "Participating Facilities" then catn=1;
if category = "Overall Agreement Status" then catn=2;
if category = "Case Determination Outcome" then catn=3;
if category = "Cases Incorrectly Reported as HAI" then catn=4;
if category = "Cases Correctly Not Reported as HAI" then catn=5;
if category = "Identified HAIs by Reporting Status" then catn=6;
if category = "Reason(s) for Disagreement^" then catn=7;
run;

*Getting lengths to make report columns more efficiently spaced;
data gettinglength_de;
set forreport_de;
catLength=length(category);
measlength=length(measure);
run;
proc sql; create table maxlength_de as select max(catlength) as maxcatlength,
max(measlength) as maxmeaslength
from gettinglength_de; quit;
data _null_;
set maxlength_de;
call symput("catlength",(maxcatlength-3));
call symput("measlength",(maxmeaslength-3));
run;
%put &catlength;
%put &measlength;

*Adding below from PABAK paper;
proc format ; 
value rating 
 0 = "poor"
 1 = "slight"
 2 = "fair"
 3 = "moderate"
 4 = "substantial"
 5 = "almost perfect"
 6 = "cannot calculate kappa" ; 
run;
*Landis, J. R., & Koch, G. G. (1977). The measurement of observer agreement for categorical data. Biometrics, 33, 159-174.;

*Calculating kappa & PABAK;

*Portions of code below are adapted from Cunningham et al:
https://support.sas.com/resources/papers/proceedings09/242-2009.pdf;
data reliability_de;
retain kappa kapstrength se_kappa LL_95_CI UL_95_CI se_kappa_null z p po pe ppos pneg 
pi bi pabak pabakstrength;
 a = &a; 
 b = &b; 
 c = &c; 
 d = &d; 
 N = a+b+c+d;
*po=The proportion of observed agreement, 
 pe=the expected proportion of agreement,
 ppos=the proportion of positive agreement,
 pneg=the proportion of negative agreement;
po = (a+d)/N ; 
 pe = ((a+c)*(a+b)+(b+d)*(c+d))/N**2 ; 
 ppos = (2*a)/(N+a-d) ; 
 pneg = (2*d)/(N-a+d) ;
pi = (a-d)/N ; 
 bi = (b-c)/N ; 
 pabak = 2*po-1 ;

*pi=prevalence index,bi=bias index, PABAK=prev adj bias adj kappa;
 pi = (a-d)/N ; 
 bi = (b-c)/N ; 
 pabak = 2*po-1 ; 

*kappa statistic and its asymptotic standard error (Fleiss, 2003);
 kappa = (po-pe)/(1-pe) ; 
 q = ((a/N)*(1-(((a+b)/N)+((a+c)/N))*(1-kappa))**2)+((d/N)* 
(1-(((c+d)/N)+((b+d)/N))*(1-kappa))**2); 
 r = ((1-kappa)**2)*((b/N)*(((a+c)/N)+((c+d)/N))**2+(c/N)* 
(((b+d)/N)+((a+b)/N))**2) ; 
 s = (kappa - pe*(1-kappa))**2 ;

*Asymptotic standard error ;
 se_kappa = sqrt((q+r-s)/(N*(1-pe)**2)) ; 
 LL_95_CI = kappa-1.96*se_kappa ; 
if LL_95_CI < -1.00 then LL_95_CI = -1.00 ; 
 UL_95_CI = kappa+1.96*se_kappa ; 
if UL_95_CI > 1 then UL_95_CI = 1.00 ;

*The 95% lower and upper confidence limits around kappa are calculated and set to -1 or +1 if the confidence limit is 
less than -1 or greater than +1. Had you specified the KAPPA option in the TABLE statement of a typical PROC 
FREQ, you would obtain these same values of kappa, the asymptotic standard error, and the 95% confidence limits. 
This program also provides a test of the null hypothesis kappa = 0 versus kappa > 0. The standard error under the 
null, the z-statistic, and the p-value are calculated (Rosner, 2000);
 se_kappa_null =sqrt((1/(N*(1-pe)**2))*(pe+(pe**2)- 
((((a+b)/N)*((a+c)/N)*(((a+b)/N)+((a+c)/N))+((c+d)/N)*((b+d)/N)*(((c+d)/ 
N)+((b+d)/N)))))) ; 
 z = kappa/se_kappa_null ; 
 p = 1 - cdf("Normal" , z, 0, 1) ;

 *Creating labels;
label se_kappa = "Kappa Std. Error"  
 LL_95_CI = "95% CI Lower Limit"  
 UL_95_CI = "95% CI Upper Limit"  
 se_kappa_null = "Kappa Std. Error (Under Ho)"  
 z = "Z (Under Ho:Kap=0)"  
 p = "One sided p-value (Under Ho:Kap=0)"  
 po = "Observed Agreement (Po)"  
 pe = "Expected Agreement (Pe)"  
 ppos = "Positive Agreement (Ppos)"  
 pneg = "Negative Agreement (Pneg)"  
 pi = "Prevalence Index"  
 bi = "Bias Index"  
 kappa = "Kappa"  
 pabak = "PABAK" ;

*Creating a strength of agreement variable to assign the RATINGS format to the value of kappa;
KapStrength = 0 ; 
if kappa gt 0.00 and kappa le 0.20 then KapStrength = 1 ; 
if kappa gt 0.20 and kappa le 0.40 then KapStrength = 2 ; 
if kappa gt 0.40 and kappa le 0.60 then KapStrength = 3 ; 
if kappa gt 0.60 and kappa le 0.80 then KapStrength = 4 ; 
if kappa gt 0.80 and kappa le 1.00 then KapStrength = 5 ; 
if kappa = . then KapStrength = 6 ; 

*Creating strength var for PABAK also;
pabakStrength = 0; 
if pabak gt 0.00 and pabak le 0.20 then pabakStrength = 1 ; 
if pabak gt 0.20 and pabak le 0.40 then pabakStrength = 2 ; 
if pabak gt 0.40 and pabak le 0.60 then pabakStrength = 3 ; 
if pabak gt 0.60 and pabak le 0.80 then pabakStrength = 4 ; 
if pabak gt 0.80 and pabak le 1.00 then pabakStrength = 5 ; 
if pabak = . then pabakStrength = 6 ; 

format po pe ppos pneg pi bi kappa se_kappa LL_95_CI UL_95_CI se_kappa_null p 
pabak 6.4 z 4.2 kapstrength pabakstrength rating.;
keep kappa kapstrength se_kappa LL_95_CI UL_95_CI se_kappa_null z p po pe ppos pneg 
pi bi pabak pabakstrength;
run;

proc transpose data= reliability_de out=reliabilitytrans_de;
run;

data reliability2_de;length Measure StrengthAgree $22.;
set reliabilitytrans_de;
if strip(_name_) = "kappa" then do;
Value = round(col1, .001); Measure=_label_;cat=1; end;
if index(_name_, "LL_") > 0 then do; 
CI_lower=round(col1,.001); cat=1; end;
if index(_name_, "UL_") > 0 then do; 
CI_upper=round(col1,.001); cat=1; end;
if index(_label_, "p-value") > 0 then do; 
pvalue=round(col1,.001); cat=1; end;
if index(_name_, "kapstrength") > 0 then do; 
StrengthAgree=propcase(put(col1,rating.)); cat=1; end;
if strip(_name_) = "pi" then do;
Value = round(col1, .001); Measure=_label_; cat=2; end;
if strip(_name_) = "bi" then do;
Value = round(col1, .001); Measure=_label_; cat=3; end;
if strip(_name_) = "pabak" then do;
Value = round(col1, .001); Measure=cats(_label_,"*");cat=4; end;
if index(_name_, "pabakstrength") > 0 then do; 
StrengthAgree=propcase(put(col1,rating.)); cat=4; end;
run;

proc sql; create table reliability_final_de as select max(measure) as Measure,
max(Value) as Value, max(StrengthAgree) as StrengthAgree, max(CI_lower) as CI_lower, 
max(CI_upper) as CI_upper, max(pvalue) as pvalue
from reliability2_de 
where cat ne .
group by cat ;
quit;

proc sort data=de_freqs; by descending Validator descending fac; run;


*Creating report;
ods excel file = "&reppath" 
options(sheet_interval="none" embedded_titles="yes" 
sheet_name = "Agreement & Reliability" 
absolute_column_width="20,7,21,12,12,7");options missing=0 ;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype = "Total";
title1 c=black h=11pt j=center "&jurname Dialysis Event Validation Summary";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "Overall Percent Agreement = %SYSFUNC(strip(&overallpa))%"; endcomp;
compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 1. Dialysis Event - Overall Validation Agreement"; endcomp;
run;
title1;
title2;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype="IVAM";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "IVAM Percent Agreement = %SYSFUNC(strip(&ivampa))%"; endcomp;
	compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 2. IVAM Event Validation"; endcomp;
run;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype="PBC";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "PBC Percent Agreement = %SYSFUNC(strip(&pbcpa))%"; endcomp;
	compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 3. PBC Event Validation"; endcomp;
run;
proc report data=de_freqs nowd 
style(column)=[just=left frame=box bordercolor=black ] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black] 
style(lines)=[just=left fontsize=7pt color=black tagattr="wrap:no" bordercolor=white];
where eventtype="PRS";
column ("Facility" fac) ("Validator" (validator, count ("Total" (count=all))));
define fac / group "" order=data;
define validator /group across order=data "" format=yesno.;
define count / missing "" ;
define all / missing "" ;
RBREAK AFTER /summarize style=[color=black];
	compute after;
	fac="Total";
	line "PRS Percent Agreement = %SYSFUNC(strip(&prspa))%"; endcomp;
	compute before _page_ /style=[background=white color=black tagattr="wrap:no" fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 4. PRS Event Validation"; endcomp;
run;
  footnote;

proc report data=reliability_final_de /*nowd*/
style(column)=[just=left frame=box bordercolor=black] 
style(header)=[background=lightblue color=black fontsize=9pt bordercolor=black ];
title1;
column (Measure Value StrengthAgree ("95% Confidence Limit" (CI_lower CI_upper)) pvalue);
define strengthagree / "Strength of Agreement^";
define ci_lower / "Lower Limit";
define ci_upper / "Upper Limit";
define pvalue / "p-value";
compute measure;
	if measure in ("Prevalence Index" "Bias Index") then call define(_col_, 'style',"style=[pretext='     ' asis=on]");
endcomp;
compute before _page_ /style=[background=white color=black fontsize=10pt bordercolor=white bordertopcolor=white just=left];
	line "Table 5. Dialysis Event - Overall Reliability"; endcomp;
compute after/style=[background=white color=black fontsize=7pt tagattr="wrap:yes" cellheight=18pt bordercolor=white bordertopcolor=white just=left];
	line "* Prevalence Adjusted Bias Adjusted Kappa";
	line "^ Strength of agreement criteria defined by Landis JR, Koch GG. (1977):";
	line "  The measurement of observer agreement for categorical data. Biometrics 33:159-174."; 
endcomp;
options missing= "";
run;
*Summary Table;
ods excel options(sheet_name = "Summary Table" sheet_interval="now" 
embedded_titles="yes" embedded_footnotes="yes" 
absolute_column_width="&catlength, &measlength, 5, 6");
footnote1 c=black h=7pt j=left "Percentages calculated from Total Cases Reviewed unless otherwise noted.";
footnote2 c=black h=7pt j=left "* Percentage calculated from Total HAIs identified by Validator";
footnote3 c=black h=7pt j=left "^ Percentage calculated from Facility-Validator Disagree";
footnote4 c=black h=7pt j=left "Report created %SYSFUNC(strip(%SYSFUNC(PUTN(%SYSFUNC(DATE()),WEEKDATE29.)))).";
proc tabulate data=forreport_de order=data;
title1 c=black h=10pt j=left "Table 6. Dialysis Event - Overall Validation Summary";
class  category/ style=[background=white foreground=black fontweight=bold] ;
class measure /style=[background=white foreground=black fontweight=light] ;
var count /style=[background=lightblue foreground=black bordercolor=black];
var percent/style=[background=lightblue foreground=black bordercolor=black];
table category="" * measure="", count="Count"*([style=[just=center bordercolor=black]] [f=4.])  
	percent="%"*([style=[just=center bordercolor=black]] [f=4.1]) / 
box=[style=[background=lightblue foreground=black bordercolor=black]] box="Measure" 
misstext= "--" ;
keylabel sum=" " ;
classlev Category / style=[background=white foreground=black fontweight=bold bordercolor=black tagattr="wrap:no" width=100%];
classlev Measure / style=[background=white foreground=black fontweight=light bordercolor=black tagattr="wrap:no" width=100%];
options missing=0;
run;
ods excel close;
options missing=.;